/************ 

   - Define HIFI parameters 
   - Calculates tau1/tau2 for HIFI or regular experiments
   - Insert the following statement in the main program :
	#include <HIFI.h>
     at the end of the parameter declaration lines

**************/

    char 	HIFI[MAXSTR];

    int         T2incr,T2counter;

    double      csa, sna,
                tilt = M_PI*getval("tilt")/180.0;

    getstr("HIFI",HIFI);
/* need to get f1180/f2180 value here; delete
   the corresponding lines in main program */
    getstr("f1180",f1180);
    getstr("f2180",f2180);


/* check HIFI parameters */

    if ( HIFI[A]!='n' && HIFI[A]!='t' && HIFI[A]!='s' )
      { text_error("Incorrect HIFI flag! HIFI should be either 'n','t' or 's' ");        
        psg_abort(1);}

    if ( HIFI[A]=='t' && (tilt==0 || tilt==90) )
      { text_error("HIFI='t' only works for tilt > 0 and < 90");
        psg_abort(1);}

    if ( (HIFI[A]=='t' || HIFI[A]=='s') && ni2>1)
      { text_error("ni2 is not used for HIFI experiments (only ni), set ni2=0");
        psg_abort(1);}


/* calculate t1_counter, t2_counter 
   note: t2_counter remains zero for HIFI experiments */

    if( ix == 1) d2_init = d2;
    t1_counter = (int) ( (d2-d2_init)*sw1 + 0.5 );

    if( ix == 1) d3_init = d3;
    t2_counter = (int) ( (d3-d3_init)*sw2 + 0.5 );


/* calculate tau1, tau2 */

    if (HIFI[A]=='t')	/* use tilt angle to recalculate sw1 and sw2 */
      {
       csa = cos(tilt);
       sna = sin(tilt);
      }
     else		/* combine sw1/sw2 as they are */
      {
       csa = 1.0;
       sna = 1.0;
      }


    if ((HIFI[A]=='t') || (HIFI[A]=='s'))	/* HIFI experiment */
      {
       sw1=sw1/csa;
       sw2=sw2/sna;

       tau1 = t1_counter/sw1;
       tau2 = t1_counter/sw2;

       T2incr=ni;		/*used in semi-constant time calculation*/
       T2counter=t1_counter;	/*used in semi-constant time calculation*/

       if ((f1180[A] == 'y') && (ni > 1.0))
         {
          tau1 += (1.0 / (2.0*sw1)); if(tau1 < 0.2e-6) tau1 = 0.0;
          tau2 += (1.0 / (2.0*sw2)); if(tau2 < 0.2e-6) tau2 = 0.0; 
          strcpy(f2180,"y");	/*used in semi-constant time calculation*/
         }
        else 
          strcpy(f2180,"n");	/*used in semi-constant time calculation*/
      }
     else	/* REGULAR experiment */
      {
       tau1 = d2;
       tau2 = d3;

       T2incr=ni2;		/*used in semi-constant time calculation*/
       T2counter=t2_counter;	/*used in semi-constant time calculation*/

       if ((f1180[A] == 'y') && (ni > 1.0))
         { tau1 += ( 1.0 / (2.0*sw1) ); if(tau1 < 0.2e-6) tau1 = 0.0; }
       if ((f2180[A] == 'y') && (ni2 > 1.0))
         { tau2 += ( 1.0 / (2.0*sw2) ); if(tau2 < 0.2e-6) tau2 = 0.0; }
      }

    tau1 = tau1/2.0;
    tau2 = tau2/2.0;

